unit Clipmap;
{
  By Daniel Parnell.
  s921878@minyos.its.rmit.edu.au
  http://minyos.its.rmit.edu.au/~s921878
}
interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs;

type
  TClipBitmap = class(TGraphicControl)
  private
    { Private declarations }
    fBitmap1,
    fBitmap2,
    fClipMap,
    TempMap  : TBitmap;

    procedure SetBitmap1(Bitmap : TBitmap);
    procedure SetBitmap2(Bitmap : TBitmap);
    procedure SetClipMap(Bitmap : TBitmap);
    procedure HasChanged(sender : TObject);
  protected
    { Protected declarations }
  public
    { Public declarations }
    constructor Create(AOwner : TComponent); override;
    procedure Paint; override;
    procedure Calculate;
    procedure Loaded; override;
    destructor Destroy; override;
  published
    { Published declarations }
    property top;
    property left;
    property width default 32;
    property height default 32;
    property Bitmap1:TBitmap read fBitmap1 write SetBitmap1;
    property Bitmap2:TBitmap read fBitmap2 write SetBitmap2;
    property Mask   :TBitmap read fClipMap write SetClipMap;
    property OnClick;
    property OnMouseDown;
    property OnMouseUp;
    property OnMouseMove;
  end;

procedure Register;

implementation

constructor TClipBitmap.Create(AOwner : TComponent);
begin
  inherited Create(AOwner);

  fBitmap1:=TBitmap.Create;
  fBitmap2:=TBitmap.Create;
  fClipMap:=TBitmap.Create;
  TempMap:=TBitmap.Create;

  fBitmap1.OnChange:=HasChanged;
  fBitmap2.OnChange:=HasChanged;
  fClipMap.OnChange:=HasChanged;

  width:=32;
  height:=32;
end;

procedure TClipBitmap.SetBitmap1(Bitmap : TBitmap);
begin
  fBitmap1.Assign(Bitmap);
  Calculate;
  Invalidate;
end;

procedure TClipBitmap.SetBitmap2(Bitmap : TBitmap);
begin
  fBitmap2.Assign(Bitmap);
  Calculate;
  Invalidate;
end;

procedure TClipBitmap.SetClipMap(Bitmap : TBitmap);
begin
  fClipMap.Assign(Bitmap);
  Calculate;
  Invalidate;
end;

procedure TClipBitmap.Paint;
begin
  if tempMap.Empty or (csDesigning in ComponentState) then
    Calculate;

  canvas.Draw(0,0,TempMap);
end;

procedure TClipBitmap.Calculate;
var
  bm1,
  bm2 : TBitmap;

  function Max(a,b : longint):longint;
  begin
    if a>b then
      result:=a
    else
      result:=b;
  end;

begin
  width:=Max(fBitmap1.width,fBitmap2.width);
  height:=Max(fBitmap1.height,fBitmap2.height);

  if width=0 then
    width:=32;
  if height=0 then
    height:=32;

  TempMap.width:=width;
  TempMap.height:=height;

  bm1:=TBitmap.Create;
  bm2:=TBitmap.Create;

  bm1.Assign(fBitmap1);
  bm2.Assign(fBitmap2);

  bm1.Canvas.CopyMode:=cmSrcAnd;
  bm1.Canvas.CopyRect(ClientRect,fClipMap.Canvas,ClientRect);

  bm2.Canvas.CopyMode:=cmSrcCopy;
  bm2.Canvas.CopyRect(ClientRect,fClipMap.Canvas,ClientRect);
  bm2.Canvas.CopyMode:=cmSrcErase;
  bm2.Canvas.CopyRect(ClientRect,fBitmap2.Canvas,ClientRect);

  tempMap.Canvas.CopyMode:=cmSrcCopy;
  tempMap.Canvas.CopyRect(ClientRect,bm1.Canvas,ClientRect);
  tempMap.Canvas.CopyMode:=cmSrcPaint;
  tempMap.Canvas.CopyRect(ClientRect,bm2.Canvas,ClientRect);

  bm1.Free;
  bm2.Free;
end;

procedure TClipBitmap.Loaded;
begin
  inherited Loaded;
  Calculate;
  Invalidate;
end;

destructor TClipBitmap.Destroy;
begin
  TempMap.Free;

  inherited Destroy;
end;

procedure TClipBitmap.HasChanged(sender : TObject);
begin
  Calculate;
  Invalidate;
end;

procedure Register;
begin
  RegisterComponents('Samples', [TClipBitmap]);
end;

end.
